{{ extend 'layout.html' }}

{{ block header }}
{{ super }}
<div>header2</div>
{{ end }}

<ul class="posts">
{{ for post in posts: }}
    <li>
        <h2>{{ =post['title'] }}</h2>
        <hr />
    </li>
{{ pass }}
</ul>
