<!DOCTYPE html>
<html>
    <head>
        <title>Test</title>
        {{ block head }}{{ end }}
    </head>
    <body>
        {{ block header }}
        {{ include '_header.html' }}
        {{ end }}
        <div class="page">
            <a href="/" class="title"><h1>Test</h1></a>
            <div class="nav">
                <a href="/">{{ ="nuvolosità variabile" }}</a>
            </div>
            {{ block main }}
            {{ include }}
            {{ end }}
        </div>
        {{ block footer }}
        {{ include './_footer.html' }}
        {{ end }}
    </body>
</html>
